# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2020-01-08 23:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0012_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailmessage',
            name='external_id',
            field=models.CharField(blank=True, db_index=True, max_length=250, null=True, verbose_name='external ID'),
        ),
    ]
