# Generated by Django 2.2.15 on 2020-12-17 14:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0020_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='dialermessage',
            name='priority',
            field=models.PositiveSmallIntegerField(default=3, verbose_name='priority'),
        ),
        migrations.AddField(
            model_name='emailmessage',
            name='priority',
            field=models.PositiveSmallIntegerField(default=3, verbose_name='priority'),
        ),
        migrations.AddField(
            model_name='outputsmsmessage',
            name='priority',
            field=models.PositiveSmallIntegerField(default=3, verbose_name='priority'),
        ),
        migrations.AddField(
            model_name='pushnotificationmessage',
            name='priority',
            field=models.PositiveSmallIntegerField(default=3, verbose_name='priority'),
        ),
    ]
