# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-22 16:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('pymess', '0005_push_notifications'),
    ]

    operations = [
        migrations.AddField(
            model_name='dialertemplate',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AddField(
            model_name='smstemplate',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AlterField(
            model_name='dialermessage',
            name='is_final_state',
            field=models.BooleanField(default=False, verbose_name='is final state'),
        ),
        migrations.AlterField(
            model_name='pushnotificationtemplate',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
    ]
